"""Winget plugin package for Porringer.

This package contains the implementation of the Winget environment plugin,
providing functionalities for installing, uninstalling, upgrading, and searching packages.
"""
